class BenchmarkConfiguration < ActiveRecord::Base

  has_many :benchmark_batches, dependent: :destroy, inverse_of: :benchmark_configuration
  belongs_to :language, inverse_of: :benchmark_configurations
  belongs_to :runtime, inverse_of: :benchmark_configurations
  belongs_to :operating_system, inverse_of: :benchmark_configurations

  scope :with_preloaded_environment_description, -> do
    includes(:operating_system, :runtime, :language)
  end

  def self.from_json(data)
    lang = find_or_create_by_name(Language, data["language"])
    runtime = find_or_create_by_name(lang.runtimes, data["runtime"])
    os = find_or_create_by_name(OperatingSystem, data["os"])
    desc = data["description"] || ""

    configuration_params = { language: lang,
                             runtime: runtime,
                             operating_system: os,
                             description: desc }
    configuration = find_by(configuration_params)
    configuration = create!(configuration_params) unless configuration

    configuration
  end

  def self.find_for_results_json(configuration_id)
    with_preloaded_environment_description.find(configuration_id)
  end

  def self.for_benchmark_infos(info_ids)
    joins(:benchmark_batches).
      joins(benchmark_batches: :benchmark_results).
      where(benchmark_results: { benchmark_info_id: info_ids }).
      distinct.
      with_preloaded_environment_description
  end

  def name
    xlated_os = operating_system.name_normalized
    full_name = "#{runtime.name} (#{language.name}) on #{xlated_os}"

    if description.present?
      "#{description} - #{full_name}"
    else
      full_name
    end
  end

  def all_benchmark_infos
    benchmark_batches.
      includes(benchmark_results: [ :benchmark_info ]).
      map(&:all_benchmark_infos).
      flatten.
      uniq
  end

  def all_dates_when_performed
    SortedSet.new(benchmark_batches.map(&:performed_at).flatten).to_a
  end

  def to_results_json
    all_benchmark_infos.map do |info|
      info.to_results_json_for_configuration(self)
    end
  end

  def self.find_or_create_by_name(ctx, name)
    inst = ctx.where(name: name).first
    inst = ctx.create!(name: name) unless inst
    inst
  end
end

# == Schema Information
#
# Table name: benchmark_configurations
#
#  id                  :integer          not null, primary key
#  name                :string
#  language_id         :integer
#  runtime_id          :integer
#  operating_system_id :integer
#  created_at          :datetime
#  updated_at          :datetime
#
